/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#include <src/investigation/donnees/page.h>

#include <src/investigation/donnees/requete.h>
#include <src/investigation/donnees/liens.h>
#include <src/investigation/donnees/compteurs.h>

using namespace Investigation::Donnees;

Page::Page(const bool brouillon, const std::vector<std::string>& identifiants, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position)
	:Instruction(position), _brouillon(brouillon), _identifiants(identifiants.begin(),identifiants.end()), _contenu(contenu) { }
Page::Page(const bool brouillon, const std::vector<std::string>& identifiants, const std::string& titre, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position)
	:Instruction(position), _brouillon(brouillon), _identifiants(identifiants.begin(),identifiants.end()), _titre(titre), _contenu(contenu) { }
Page::Page(const bool brouillon, const std::set<std::string>& identifiants, const std::string& titre, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position, const Identifiant& identifiant)
	:Instruction(position), _brouillon(brouillon), _identifiants(identifiants), _titre(titre), _contenu(contenu), _identifiant(identifiant) { }
Page::~Page() {}
Investigation::Donnees::SequencePages Page::operator() (const Investigation::Donnees::Requete& requete) const
{
	Investigation::Donnees::SequencePages resultat;
	std::string motclef = requete;
	if(_identifiants.find(motclef)!=_identifiants.end())
	{
		PageSP page(new Page(*this));
		resultat.push_back(page);
	}
	return resultat;
}
InstructionSP Page::clone() const
{
	SequenceContenuPage contenu;
	for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
	{
		contenu.push_back((*it)->clone());
	}
	InstructionSP page(new Page(_brouillon,_identifiants,_titre,contenu,_position,_identifiant));
	return page;
}
std::set<std::string> Page::operator[] (const Investigation::Donnees::Requete& requete)
{
	std::set<std::string> mc;
	if(not requete or (requete and (_identifiants.find(requete)!=_identifiants.end())))
	{
		_statistiques(true);
		for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
		{
			auto itmc = (**it)[requete];
			mc.insert(itmc.cbegin(),itmc.cend());
		}
	}
	else
	{
		_statistiques(false);
	}
	return mc;
}
bool Page::operator%(const Instruction& instruction) const
{
	const Page& page=dynamic_cast<const Page&>(instruction); // throw bad_cast
	return (*this)%(page);
}
bool Page::operator%(const Page& page) const
{
	if(_statistiques!=page._statistiques)
		return false;
	if(_contenu.size()!=page._contenu.size())
		throw;
	for(auto it=_contenu.cbegin(), itp=page._contenu.cbegin() ; it!=_contenu.cend() ; ++it,++itp)
	{
		if(not ((**it)%(**itp)))
			return false;
	}
	return true;
}

Html::Div Page::operator() (const std::string& methode, const Investigation::Donnees::Requete& requete, Investigation::Donnees::Liens& liens) const
{
	Html::Div div_page(_brouillon?"investigation page draft":"investigation page");
	Html::Hr hr_page("investigation page");
	Html::H titre_page(2,"investigation page title");
	titre_page << Html::Texte(_titre);
	div_page << titre_page;
	for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
	{
		div_page << (**it)(methode,requete,liens);
	}
	return div_page;
}
std::set<std::string> Page::identifiants() const
{
	return _identifiants;
}
void Page::liens_valides(const std::set<std::string>& identifiants) const
{
	for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
	{
		(*it)->liens_valides(identifiants);
	}
}
void Page::propage_infos()
{
	for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
	{
		(*it)->propage_infos(_titre);
	}
}
void Page::compte(CompteursSite& compteurs) const
{
	compteurs += (*this);
	for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
	{
		(*it)->compte(compteurs);
	}
}
long int Page::niveau_information() const
{
	long int niveau_information = 0;
	for(auto it=_contenu.cbegin() ; it!=_contenu.cend() ; ++it)
	{
		niveau_information += (*it)->niveau_information();
	}
	return niveau_information;
}
